"use strict";
// @ts-check
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const ErrorHJDB_1 = __importDefault(require("./Utils/ErrorHJDB"));
const create_database_1 = __importDefault(require("./Utils/create_database"));
const fs_1 = require("fs");
const read_file_1 = __importDefault(require("./Utils/read_file"));
const is_empty_1 = __importDefault(require("./Utils/is_empty"));
class hyperJDB {
    /**
     * @constructor Creates a database
     * @param {string} name The name of your database
     */
    constructor(name) {
        if ((0, is_empty_1.default)(name))
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a name to your database");
        this.name = name;
        this.path_database = (0, create_database_1.default)(name);
        this.databasesObj = {};
    }
    /**
     * Sets a value into the database
     * @param {string} key JSON key to store data
     * @param {any} value The value to be stored
     * @returns {true}
     */
    set(key, value) {
        if ((0, is_empty_1.default)(key))
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a key to store the value");
        if (!value && value != 0)
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a value to be stored in the database");
        let response = (0, read_file_1.default)(this.path_database);
        this.databasesObj[this.name] = response;
        let jsonRoute = key.replace(/\[/g, ".").replace(/\]/g, "").split(".");
        let jsonObject = this.databasesObj[this.name];
        for (let i = 0; i < jsonRoute.length; i++) {
            const prop = jsonRoute[i];
            if (i === jsonRoute.length - 1) { /* Last iteration of the loop */
                jsonObject[prop] = value; /* Set given value */
            }
            else {
                jsonObject[prop] = jsonObject[prop] || {}; /* Initialize sub-object if it doesn't exist */
            }
            jsonObject = jsonObject[prop];
        }
        try {
            (0, fs_1.writeFileSync)(this.path_database, JSON.stringify(this.databasesObj[this.name], null, 2), 'utf-8');
        }
        catch (err) {
            throw new ErrorHJDB_1.default("Writing error", "There was an error updating your database");
        }
        return true;
    }
    /**
     * Gets an existing key from the database.
     * @param {string} key JSON key to be searched
     * @returns {any} The stored value
     */
    get(key) {
        if ((0, is_empty_1.default)(key))
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a key to be searched in the database");
        let response = (0, read_file_1.default)(this.path_database);
        this.databasesObj[this.name] = response;
        let jsonRoute = key.replace(/\[/g, ".").replace(/\]/g, "").split(".");
        let jsonObject = this.databasesObj[this.name];
        for (const prop of jsonRoute) {
            jsonObject = jsonObject[prop];
        }
        return jsonObject;
    }
    /**
     * Deletes an existing key from the database.
     * @param {string} key JSON key to search and delete
     * @returns {true}
     */
    delete(key) {
        if ((0, is_empty_1.default)(key))
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a key to be deleted from the database");
        let response = (0, read_file_1.default)(this.path_database);
        this.databasesObj[this.name] = response;
        let jsonRoute = key.replace(/\[/g, ".").replace(/\]/g, "").split(".");
        let jsonObject = this.databasesObj[this.name];
        for (const prop of jsonRoute) {
            if (prop == jsonRoute.slice(-1)[0]) { /* Last iteration of the loop */
                if (Object.prototype.toString.call(jsonObject) == '[object Array]') { /* Array */
                    jsonObject = jsonObject.splice(prop, 1);
                }
                else if (!jsonObject[prop] && jsonObject[prop] != 0) { /* No value */
                    return false;
                }
                else { /* Value */
                    delete jsonObject[prop];
                }
            }
            jsonObject = jsonObject[prop];
        }
        try {
            (0, fs_1.writeFileSync)(this.path_database, JSON.stringify(this.databasesObj[this.name], null, 2), 'utf-8');
        }
        catch (err) {
            throw new ErrorHJDB_1.default("Writing error", "There was an error updating your database");
        }
        return true;
    }
    /**
     * Searches the given key.
     * @param {string} key The key to be searched
     * @returns {boolean} True if found, false if not
     */
    has(key) {
        if ((0, is_empty_1.default)(key))
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a key to be searched in the database");
        let response = (0, read_file_1.default)(this.path_database);
        this.databasesObj[this.name] = response;
        let jsonRoute = key.replace(/\[/g, ".").replace(/\]/g, "").split(".");
        let jsonObject = this.databasesObj[this.name];
        var last = false;
        for (const prop of jsonRoute) {
            if (prop == jsonRoute.slice(-1)[0]) { /* Last iteration of the loop */
                if (!jsonObject[prop] && jsonObject[prop] != 0) {
                    last = false;
                }
                else {
                    last = true;
                }
            }
            jsonObject = jsonObject[prop];
        }
        return last;
    }
    /**
     * Pushes to the given array a new value.
     * @param {string} key
     * @param {any} value
     * @returns The new array
     */
    push(key, value) {
        if ((0, is_empty_1.default)(key))
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a key to be searched in the database");
        if (!value && value != 0)
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a value to be stored in the database");
        let response = (0, read_file_1.default)(this.path_database);
        this.databasesObj[this.name] = response;
        let jsonRoute = key.replace(/\[/g, ".").replace(/\]/g, "").split(".");
        let jsonObject = this.databasesObj[this.name];
        var last = [];
        for (const prop of jsonRoute) {
            if (prop == jsonRoute.slice(-1)[0]) { /* Last iteration of the loop */
                if (!(Object.prototype.toString.call(jsonObject[prop]) == '[object Array]'))
                    throw new ErrorHJDB_1.default("Invalid type", `Stored value: \`${jsonObject[prop]}\` is not an Array`);
                jsonObject[prop].push(value);
                last = jsonObject[prop];
            }
            jsonObject = jsonObject[prop];
        }
        try {
            (0, fs_1.writeFileSync)(this.path_database, JSON.stringify(this.databasesObj[this.name], null, 2), 'utf-8');
        }
        catch (err) {
            throw new ErrorHJDB_1.default("Writing error", "There was an error updating your database");
        }
        return last;
    }
    /**
     * Adds to the given key the quantity provided
     * @param {string} key JSON key to be searched
     * @param {number} quantity Amount to be added
     * @returns {number} Updated value
     */
    add(key, quantity) {
        if ((0, is_empty_1.default)(key))
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a key to be operated in the database");
        if (!quantity)
            throw new ErrorHJDB_1.default("Missing argument", `You must provide a number to be added to \`${key}\``);
        if (isNaN(quantity))
            throw new ErrorHJDB_1.default("Invalid value", "The `quantity` parameter must be a number");
        let response = (0, read_file_1.default)(this.path_database);
        this.databasesObj[this.name] = response;
        let jsonRoute = key.replace(/\[/g, ".").replace(/\]/g, "").split(".");
        let jsonObject = this.databasesObj[this.name];
        var last = 0;
        for (const prop of jsonRoute) {
            if (prop == jsonRoute.slice(-1)[0]) { /* Last iteration of the loop */
                if (isNaN(jsonObject[prop]))
                    throw new ErrorHJDB_1.default("Invalid type", `Stored value: \`${jsonObject[prop]}\` is not a Number`);
                jsonObject[prop] = jsonObject[prop] + quantity;
                last = jsonObject[prop];
            }
            jsonObject = jsonObject[prop];
        }
        try {
            (0, fs_1.writeFileSync)(this.path_database, JSON.stringify(this.databasesObj[this.name], null, 2), 'utf-8');
        }
        catch (err) {
            throw new ErrorHJDB_1.default("Writing error", "There was an error updating your database");
        }
        return last;
    }
    /**
     * Substracts from the given key the quantity provided
     * @param {string} key JSON key to be searched
     * @param {number} quantity Amount to be substracted
     * @returns {number} Updated value
     */
    substract(key, quantity) {
        if ((0, is_empty_1.default)(key))
            throw new ErrorHJDB_1.default("Missing argument", "You must provide a key to be operated in the database");
        if (!quantity)
            throw new ErrorHJDB_1.default("Missing argument", `You must provide a number to be substracted from \`${key}\``);
        if (isNaN(quantity))
            throw new ErrorHJDB_1.default("Invalid value", "The `quantity` parameter must be a number");
        let response = (0, read_file_1.default)(this.path_database);
        this.databasesObj[this.name] = response;
        let jsonRoute = key.replace(/\[/g, ".").replace(/\]/g, "").split(".");
        let jsonObject = this.databasesObj[this.name];
        var last = 0;
        for (const prop of jsonRoute) {
            if (prop == jsonRoute.slice(-1)[0]) { /* Last iteration of the loop */
                if (isNaN(jsonObject[prop]))
                    throw new ErrorHJDB_1.default("Invalid type", `Stored value: \`${jsonObject[prop]}\` is not a number`);
                jsonObject[prop] = jsonObject[prop] - quantity;
                last = jsonObject[prop];
            }
            jsonObject = jsonObject[prop];
        }
        try {
            (0, fs_1.writeFileSync)(this.path_database, JSON.stringify(this.databasesObj[this.name], null, 2), 'utf-8');
        }
        catch (err) {
            throw new ErrorHJDB_1.default("Writing error", "There was an error updating your database");
        }
        return last;
    }
    /**
     * Clears **all** the data in the .json file
     * @returns {true}
     */
    drop() {
        try {
            (0, fs_1.writeFileSync)(this.path_database, JSON.stringify({}, null, 2), 'utf-8');
        }
        catch (err) {
            throw new ErrorHJDB_1.default("Writing error", "There was an error updating your database");
        }
        return true;
    }
}
module.exports = hyperJDB;
